@@ -1,3 +1,11 @@
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
 #include <sirius.hpp>
 #include <cfenv>
 #include "core/profiler.hpp"
@@ -55,7 +63,7 @@ rewrite_relative_paths(json& dict__, fs::path const& working_directory = fs::cur
     }
 }
 
-nlohmann::json
+auto
 preprocess_json_input(std::string fname__)
 {
     if (fname__.find("{") == std::string::npos) {
@@ -89,7 +97,7 @@ create_sim_ctx(std::string fname__, cmd_args const& args__)
     return ctx_ptr;
 }
 
-double
+auto
 ground_state(Simulation_context& ctx, int task_id, cmd_args const& args, int write_output)
 {
     print_memory_usage(ctx.out(), FILE_LINE);
@@ -155,7 +163,7 @@ ground_state(Simulation_context& ctx, int task_id, cmd_args const& args, int wri
         compute_forces = true;
     }
 
-    nlohmann::json result;
+    json result;
 
     Lattice_relaxation lr(dft);
 
@@ -228,8 +236,7 @@ ground_state(Simulation_context& ctx, int task_id, cmd_args const& args, int wri
         if (ctx.comm().rank() == 0) {
             std::string output_file = args.value<std::string>("output", std::string("output_") + ctx.start_time_tag() +
                                                                                 std::string(".json"));
-            std::ofstream ofs(output_file, std::ofstream::out | std::ofstream::trunc);
-            ofs << dict.dump(4);
+            write_json_to_file(dict, output_file);
         }
 
         // if (args.exist("aiida_output")) {
@@ -307,9 +314,32 @@ ground_state(Simulation_context& ctx, int task_id, cmd_args const& args, int wri
         double e2 = dict_ref["ground_state"]["energy"]["total"].get<double>();
 
         if (std::abs(e1 - e2) > 1e-5) {
-            std::printf("total energy is different: %18.7f computed vs. %18.7f reference\n", e1, e2);
+            std::cout << "total energy is different: " << e1 << " computed vs. " << e2 << " reference" << std::endl;
             ctx.comm().abort(1);
         }
+        if (result.count("magnetisation") && dict_ref["ground_state"].count("magnetisation")) {
+            double max_diff{0};
+            auto t1 = result["magnetisation"]["total"].get<std::vector<double>>();
+            auto t2 = dict_ref["ground_state"]["magnetisation"]["total"].get<std::vector<double>>();
+            for (int x : {0, 1, 2}) {
+                max_diff = std::max(max_diff, std::abs(t1[x] - t2[x]));
+            }
+            auto v1 = result["magnetisation"]["atoms"].get<std::vector<std::vector<double>>>();
+            auto v2 = dict_ref["ground_state"]["magnetisation"]["atoms"].get<std::vector<std::vector<double>>>();
+            if (v1.size() != v2.size()) {
+                std::cout << "length of atomic magnetisations is different" << std::endl;
+                ctx.comm().abort(4);
+            }
+            for (size_t i = 0; i < v1.size(); i++) {
+                for (int x : {0, 1, 2}) {
+                    max_diff = std::max(max_diff, std::abs(v1[i][x] - v2[i][x]));
+                }
+            }
+            if (max_diff > 1e-5) {
+                std::cout << "magnetisations is different!" << std::endl;
+                ctx.comm().abort(5);
+            }
+        }
         if (result.count("stress") && dict_ref["ground_state"].count("stress")) {
             double diff{0};
             auto s1 = result["stress"].get<std::vector<std::vector<double>>>();
@@ -320,7 +350,7 @@ ground_state(Simulation_context& ctx, int task_id, cmd_args const& args, int wri
                 }
             }
             if (diff > 1e-5) {
-                std::printf("total stress is different!");
+                std::cout << "total stress is different!" << std::endl;
                 std::cout << "  reference: " << dict_ref["ground_state"]["stress"] << "\n";
                 std::cout << "  computed: " << result["stress"] << "\n";
                 ctx.comm().abort(2);
@@ -336,7 +366,7 @@ ground_state(Simulation_context& ctx, int task_id, cmd_args const& args, int wri
                 }
             }
             if (diff > 1e-6) {
-                std::printf("total force is different!");
+                std::cout << "total force is different!" << std::endl;
                 std::cout << "  reference: " << dict_ref["ground_state"]["forces"] << "\n";
                 std::cout << "  computed: " << result["forces"] << "\n";
                 ctx.comm().abort(3);
@@ -347,7 +377,7 @@ ground_state(Simulation_context& ctx, int task_id, cmd_args const& args, int wri
     /* wait for all */
     ctx.comm().barrier();
 
-    return result["energy"]["total"].get<double>();
+    return result;
 }
 
 /// Run a task based on a command line input.
@@ -366,7 +396,7 @@ run_tasks(cmd_args const& args)
 
     if (!fs::exists(fpath)) {
         if (mpi::Communicator::world().rank() == 0) {
-            std::printf("input file does not exist\n");
+            std::cout << "input file does not exist" << std::endl;
         }
         return;
     }
@@ -385,8 +415,13 @@ run_tasks(cmd_args const& args)
 
         int write_output{0};
 
+        json dict;
+        json_output_common(dict);
+        // dict["context"] = ctx.serialize();
+        dict["result"] = {};
+
         int rank{0};
-        int num_steps{10};
+        int num_steps{7};
         std::vector<double> volume;
         std::vector<double> energy;
         for (int i = 0; i < num_steps; i++) {
@@ -398,14 +433,18 @@ run_tasks(cmd_args const& args)
             ctx->unit_cell().set_lattice_vectors(lv);
             ctx->initialize();
             auto e = ground_state(*ctx, task_t::ground_state_new, args, write_output);
+            dict["result"] += e;
             volume.push_back(ctx->unit_cell().omega());
-            energy.push_back(e);
+            energy.push_back(e["energy"]["free"].get<double>());
         }
         if (rank == 0) {
             std::cout << "final result:" << std::endl;
             for (int i = 0; i < num_steps; i++) {
                 std::cout << "volume: " << volume[i] << ", energy: " << energy[i] << std::endl;
             }
+            dict["volume"] = volume;
+            dict["energy"] = energy;
+            write_json_to_file(dict, "output_eos.json");
         }
     }
 
@@ -471,7 +510,8 @@ run_tasks(cmd_args const& args)
                 // saved file potential.U().calculate_hubbard_potential_and_energy(potential.U().occupation_matrix());
             }
         }
-        sirius::diagonalize<double, double>(H0, ks, ctx->cfg().iterative_solver().energy_tolerance());
+        sirius::diagonalize<double, double>(H0, ks, ctx->cfg().iterative_solver().energy_tolerance(),
+                                            ctx->cfg().iterative_solver().num_steps());
 
         ks.sync_band<double, sync_band_t::energy>();
         if (mpi::Communicator::world().rank() == 0) {
@@ -516,35 +556,33 @@ int
 main(int argn, char** argv)
 {
     std::feclearexcept(FE_ALL_EXCEPT);
-    cmd_args args;
-    args.register_key("--input=", "{string} input file name");
-    args.register_key("--output=", "{string} output file name");
-    args.register_key("--task=", "{int} task id");
-    args.register_key("--aiida_output", "write output for AiiDA");
-    args.register_key("--test_against=", "{string} json file with reference values");
-    args.register_key("--repeat_update=", "{int} number of times to repeat update()");
-    args.register_key("--fpe", "enable check of floating-point exceptions using GNUC library");
-    args.register_key("--control.processing_unit=", "");
-    args.register_key("--control.verbosity=", "");
-    args.register_key("--control.verification=", "");
-    args.register_key("--control.mpi_grid_dims=", "");
-    args.register_key("--control.std_evp_solver_name=", "");
-    args.register_key("--control.gen_evp_solver_name=", "");
-    args.register_key("--control.fft_mode=", "");
-    args.register_key("--control.memory_usage=", "");
-    args.register_key("--parameters.ngridk=", "");
-    args.register_key("--parameters.gamma_point=", "");
-    args.register_key("--parameters.pw_cutoff=", "");
-    args.register_key("--parameters.gk_cutoff=", "");
-    args.register_key("--iterative_solver.orthogonalize=", "");
-    args.register_key("--iterative_solver.early_restart=",
-                      "{double} value between 0 and 1 to control the early restart ratio in Davidson");
-    args.register_key("--mixer.type=", "{string} mixer name (anderson, anderson_stable, broyden2, linear)");
-    args.register_key("--mixer.beta=", "{double} mixing parameter");
-    args.register_key("--volume_scale0=", "{double} starting volume scale for EOS calculation");
-    args.register_key("--volume_scale1=", "{double} final volume scale for EOS calculation");
-
-    args.parse_args(argn, argv);
+    cmd_args args(argn, argv,
+                  {{"input=", "{string} input file name"},
+                   {"output=", "{string} output file name"},
+                   {"task=", "{int} task id"},
+                   {"aiida_output", "write output for AiiDA"},
+                   {"test_against=", "{string} json file with reference values"},
+                   {"repeat_update=", "{int} number of times to repeat update()"},
+                   {"fpe", "enable check of floating-point exceptions using GNUC library"},
+                   {"control.processing_unit=", ""},
+                   {"control.verbosity=", ""},
+                   {"control.verification=", ""},
+                   {"control.mpi_grid_dims=", ""},
+                   {"control.std_evp_solver_name=", ""},
+                   {"control.gen_evp_solver_name=", ""},
+                   {"control.fft_mode=", ""},
+                   {"control.memory_usage=", ""},
+                   {"parameters.ngridk=", ""},
+                   {"parameters.gamma_point=", ""},
+                   {"parameters.pw_cutoff=", ""},
+                   {"parameters.gk_cutoff=", ""},
+                   {"iterative_solver.orthogonalize=", ""},
+                   {"iterative_solver.early_restart=",
+                    "{double} value between 0 and 1 to control the early restart ratio in Davidson"},
+                   {"mixer.type=", "{string} mixer name (anderson, anderson_stable, broyden2, linear)"},
+                   {"mixer.beta=", "{double} mixing parameter"},
+                   {"volume_scale0=", "{double} starting volume scale for EOS calculation"},
+                   {"volume_scale1=", "{double} final volume scale for EOS calculation"}});
 
 #if defined(_GNU_SOURCE)
     if (args.exist("fpe")) {
@@ -567,6 +605,10 @@ main(int argn, char** argv)
         exception_thrown = true;
     }
 
+    if (exception_thrown) {
+        mpi::Communicator::world().abort(-1);
+    }
+
     sirius::finalize(1);
 
     if (my_rank == 0) {
