@@ -1,3 +1,11 @@
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
 #include <sirius.hpp>
 #include "testing.hpp"
 
@@ -46,8 +54,6 @@ main(int argn, char** argv)
 {
     cmd_args args;
 
-    args.parse_args(argn, argv);
-
     sirius::initialize(true);
     auto result = call_test(argv[0], run_test, args);
     sirius::finalize();
