@@ -1,11 +1,20 @@
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
 #include <sirius.hpp>
+#include <testing.hpp>
 #include <math.h>
 #include <complex.h>
 
 using namespace sirius;
 
 int
-run_test(cmd_args& args)
+test_sinx_cosx(cmd_args const& args)
 {
     int n{10};
 
@@ -29,24 +38,5 @@ int
 main(int argn, char** argv)
 {
     cmd_args args;
-
-    args.parse_args(argn, argv);
-
-    sirius::initialize(true);
-    printf("running %-30s : ", argv[0]);
-    int result = run_test(args);
-    if (result) {
-        printf("\x1b[31m"
-               "Failed"
-               "\x1b[0m"
-               "\n");
-    } else {
-        printf("\x1b[32m"
-               "OK"
-               "\x1b[0m"
-               "\n");
-    }
-    sirius::finalize();
-
-    return result;
+    return call_test(argv[0], test_sinx_cosx, args);
 }
