@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2019 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file simulation_context.cpp
  *
@@ -130,7 +119,11 @@ Simulation_context::init_fft_grid()
     }
 
     /* create FFT grid for coarse mesh */
-    fft_coarse_grid_ = fft::get_min_grid(2 * gk_cutoff(), rlv);
+    if (cfg().settings().use_coarse_fft_grid()) {
+        fft_coarse_grid_ = fft::get_min_grid(2 * gk_cutoff(), rlv);
+    } else {
+        fft_coarse_grid_ = fft_grid_;
+    }
 }
 
 double
@@ -482,13 +475,33 @@ Simulation_context::initialize()
 
     if (cfg().parameters().precision_wf() == "fp32" && cfg().parameters().precision_gs() == "fp64") {
         double t = std::numeric_limits<float>::epsilon() * 10;
-        auto tol = std::max(cfg().settings().itsol_tol_min(), t);
-        cfg().settings().itsol_tol_min(tol);
+        auto tol = std::max(cfg().iterative_solver().min_tolerance(), t);
+        cfg().iterative_solver().min_tolerance(tol);
     }
 
     /* set the smearing */
     smearing(cfg().parameters().smearing());
 
+    /* create auxiliary mpi grid for symmetrization */
+    auto make_mpi_grid_mt_sym = [](int na, int np) {
+        std::vector<int> result;
+        for (int ia = 1; ia <= na; ia++) {
+            if (na % ia == 0 && np % ia == 0) {
+                result = std::vector<int>({ia, np / ia});
+            }
+        }
+        return result;
+    };
+
+    for (int ic = 0; ic < unit_cell().num_atom_symmetry_classes(); ic++) {
+        if (this->full_potential() || unit_cell().atom_symmetry_class(ic).atom_type().is_paw()) {
+            auto r = make_mpi_grid_mt_sym(unit_cell().atom_symmetry_class(ic).num_atoms(), this->comm().size());
+            mpi_grid_mt_sym_.push_back(std::make_unique<mpi::Grid>(r, this->comm()));
+        } else {
+            mpi_grid_mt_sym_.push_back(nullptr);
+        }
+    }
+
     /* create G-vectors on the first call to update() */
     update();
 
@@ -809,6 +822,20 @@ Simulation_context::update()
     /* update unit cell (reciprocal lattice, etc.) */
     unit_cell().update();
 
+    if (unit_cell().num_atoms()) {
+        /* cache rotation symmetry matrices */
+        int lmax = this->full_potential() ? std::max(this->lmax_pot(), this->lmax_rho())
+                                          : std::max(2 * this->unit_cell().lmax(), 4);
+        rotm_.resize(this->unit_cell().symmetry().size());
+        /* loop over crystal symmetries */
+        #pragma omp parallel for
+        for (int i = 0; i < this->unit_cell().symmetry().size(); i++) {
+            /* compute Rlm rotation matrix */
+            rotm_[i] = sht::rotation_matrix<double>(lmax, this->unit_cell().symmetry()[i].spg_op.euler_angles,
+                                                    this->unit_cell().symmetry()[i].spg_op.proper);
+        }
+    }
+
     /* get new reciprocal vector */
     auto rlv = unit_cell().reciprocal_lattice_vectors();
 
@@ -891,7 +918,7 @@ Simulation_context::update()
                 gvec_coord_ = mdarray<int, 2>({gvec().count(), 3}, mdarray_label("gvec_coord_"));
                 #pragma omp parallel for schedule(static)
                 for (int igloc = 0; igloc < gvec().count(); igloc++) {
-                    auto G = gvec().gvec<index_domain_t::local>(igloc);
+                    auto G = gvec().gvec(gvec_index_t::local(igloc));
                     for (int x : {0, 1, 2}) {
                         gvec_coord_(igloc, x) = G[x];
                     }
@@ -924,7 +951,7 @@ Simulation_context::update()
         for (int igloc = 0; igloc < gvec().count(); igloc++) {
             int ig = gvec().offset() + igloc;
 
-            auto gv = gvec().gvec<index_domain_t::local>(igloc);
+            auto gv = gvec().gvec(gvec_index_t::local(igloc));
             /* check limits */
             for (int x : {0, 1, 2}) {
                 auto limits = fft_grid().limits(x);
@@ -932,7 +959,7 @@ Simulation_context::update()
                 if (gv[x] < limits.first || gv[x] > limits.second) {
                     std::stringstream s;
                     s << "G-vector is outside of grid limits\n"
-                      << "  G: " << gv << ", length: " << gvec().gvec_cart<index_domain_t::global>(ig).length() << "\n"
+                      << "  G: " << gv << ", length: " << gvec().gvec_cart(gvec_index_t::global(ig)).length() << "\n"
                       << "  FFT grid limits: " << fft_grid().limits(0).first << " " << fft_grid().limits(0).second
                       << " " << fft_grid().limits(1).first << " " << fft_grid().limits(1).second << " "
                       << fft_grid().limits(2).first << " " << fft_grid().limits(2).second << "\n"
@@ -943,7 +970,7 @@ Simulation_context::update()
         }
     }
 
-    if (unit_cell().num_atoms()) {
+    if (unit_cell().num_atoms() && this->num_mag_dims()) {
         init_atoms_to_grid_idx(cfg().control().rmt_max());
     }
 
@@ -1012,7 +1039,7 @@ Simulation_context::update()
         /* find the new maximum length of G-vectors */
         double new_pw_cutoff{this->pw_cutoff()};
         for (int igloc = 0; igloc < gvec().count(); igloc++) {
-            new_pw_cutoff = std::max(new_pw_cutoff, gvec().gvec_len<index_domain_t::local>(igloc));
+            new_pw_cutoff = std::max(new_pw_cutoff, gvec().gvec_len(gvec_index_t::local(igloc)));
         }
         gvec().comm().allreduce<double, mpi::op_t::max>(&new_pw_cutoff, 1);
         /* estimate new G+k-vectors cutoff */
@@ -1142,7 +1169,7 @@ Simulation_context::create_storage_file(std::string name__) const
 
         mdarray<int, 2> gv({3, gvec().num_gvec()});
         for (int ig = 0; ig < gvec().num_gvec(); ig++) {
-            auto G = gvec().gvec<index_domain_t::global>(ig);
+            auto G = gvec().gvec(gvec_index_t::global(ig));
             for (int x : {0, 1, 2}) {
                 gv(x, ig) = G[x];
             }
