@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2019 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file simulation_context.hpp
  *
@@ -296,6 +285,13 @@ class Simulation_context : public Simulation_parameters
     /// Stores all radial integrals.
     radial_integrals_t ri_;
 
+    /// MPI grid for muffin-tin symmetrization.
+    /** MPI grid is defined for each atom symmetry class */
+    std::vector<std::unique_ptr<mpi::Grid>> mpi_grid_mt_sym_;
+
+    /// Rotation matrices for real spherical harmonics.
+    std::vector<std::vector<mdarray<double, 2>>> rotm_;
+
     mutable double evp_work_count_{0};
     mutable int num_loc_op_applied_{0};
     /// Total number of iterative solver steps.
@@ -325,6 +321,7 @@ class Simulation_context : public Simulation_parameters
 
         unit_cell_ = std::make_unique<Unit_cell>(*this, comm_);
 
+        /* read from external config file provided by SIRIUS_CONFIG environment variable */
         this->import(env::config_file());
     }
 
@@ -596,7 +593,7 @@ class Simulation_context : public Simulation_parameters
     inline auto
     gvec_phase_factor(int ig__, int ia__) const
     {
-        return gvec_phase_factor(gvec().gvec<index_domain_t::global>(ig__), ia__);
+        return gvec_phase_factor(gvec().gvec(gvec_index_t::global(ig__)), ia__);
     }
 
     inline auto const&
@@ -857,6 +854,18 @@ class Simulation_context : public Simulation_parameters
         }
         return ptr;
     }
+
+    inline auto const&
+    mpi_grid_mt_sym() const
+    {
+        return mpi_grid_mt_sym_;
+    }
+
+    inline auto const&
+    rotm() const
+    {
+        return rotm_;
+    }
 };
 
 } // namespace sirius
