@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2022 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file wave_functions.hpp
  *
@@ -760,7 +749,7 @@ class Wave_functions : public Wave_functions_mt<T>
         std::vector<int> rowsplit(this->comm_.size() + 1);
         rowsplit[0] = 0;
         for (int i = 0; i < this->comm_.size(); i++) {
-            rowsplit[i + 1] = rowsplit[i] + gkvec_->gvec_count(i);
+            rowsplit[i + 1] = rowsplit[i] + gkvec_->count(i);
         }
         std::vector<int> colsplit({0, b__.size()});
         std::vector<int> owners(this->comm_.size());
@@ -850,7 +839,7 @@ struct shuffle_to
                │              │          │       │      │          │   │   │   │  │
                │              │ partial  │       │      │   full   │   │   │   │  │
                │              │  swap    │       │      │   swap   │   │   │   │  │
-   G+k index   ├──────────────┤    ->    ├───────┼──────┤    ->    ├───┼───┼───┼──┤
+   G+k index   ├──────────────┤    ->    ├───────┼──────┤    ->    │   │   │   │  │
  (distributed) │              │          │       │      │          │   │   │   │  │
                │              │          │       │      │          │   │   │   │  │
                │              │          │       │      │          │   │   │   │  │
