@@ -1,22 +1,10 @@
-// Copyright (c) 2013-2019 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file density.cpp
  *
@@ -179,7 +167,7 @@ Density::initial_density_pseudo()
     // TODO: MPI parallelise over G-shells
     auto const ff = ctx_.ri().ps_rho_->values(q, ctx_.comm());
     /* make Vloc(G) */
-    auto v = make_periodic_function<index_domain_t::local>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(), ff);
+    auto v = make_periodic_function<true>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(), ff);
 
     if (env::print_checksum()) {
         auto z1 = mdarray<std::complex<double>, 1>({ctx_.gvec().count()}, &v[0]).checksum();
@@ -224,28 +212,61 @@ Density::initial_density_pseudo()
 
     /* initialize the magnetization */
     if (ctx_.num_mag_dims()) {
-        auto Rmt = unit_cell_.find_mt_radii(1, true);
+        if (ctx_.cfg().settings().smooth_initial_mag()) {
+            /* weight function in the form exp(-alpha * r^2) for each atom;
+             * Fourier components are computed here */
+            auto gw = [&](double g) {
+                const double alpha{4.0};
+                if (g < 1e-10) {
+                    return 1.0;
+                } else {
+                    return std::exp(-std::pow(g, 2) / 4.0 / alpha);
+                }
+            };
 
-        /* auxiliary weight function; the volume integral of this function is equal to 1 */
-        auto w = [](double R, double x) {
-            double norm = 3.1886583903476735 * std::pow(R, 3);
+            double oi = 1.0 / unit_cell_.omega();
+            #pragma omp parallel for
+            for (int igloc = 0; igloc < ctx_.gvec().count(); igloc++) {
+                int ig   = ctx_.gvec().offset() + igloc;
+                double g = ctx_.gvec().gvec_len(gvec_index_t::local(igloc));
+                double f = gw(g) * oi;
+                for (int ia = 0; ia < unit_cell_.num_atoms(); ia++) {
+                    auto ff = std::conj(ctx_.gvec_phase_factor(ig, ia)) * f;
+                    auto v  = unit_cell_.atom(ia).vector_field();
+                    mag(0).rg().f_pw_local(igloc) += v[2] * ff;
+                    if (ctx_.num_mag_dims() == 3) {
+                        mag(1).rg().f_pw_local(igloc) += v[0] * ff;
+                        mag(2).rg().f_pw_local(igloc) += v[1] * ff;
+                    }
+                }
+            }
+            for (int j = 0; j < ctx_.num_mag_dims(); j++) {
+                mag(j).rg().fft_transform(1);
+            }
+        } else {
+            auto Rmt = unit_cell_.find_mt_radii(1, true);
 
-            return (1 - std::pow(x / R, 2)) * std::exp(x / R) / norm;
-        };
+            /* auxiliary weight function; the volume integral of this function is equal to 1 */
+            auto w = [](double R, double x) {
+                double norm = 3.1886583903476735 * std::pow(R, 3);
 
-        for (int ia = 0; ia < unit_cell_.num_atoms(); ia++) {
-            auto& atom_to_grid_map = ctx_.atoms_to_grid_idx_map(ia);
+                return (1 - std::pow(x / R, 2)) * std::exp(x / R) / norm;
+            };
 
-            auto v = unit_cell_.atom(ia).vector_field();
+            for (int ia = 0; ia < unit_cell_.num_atoms(); ia++) {
+                auto& atom_to_grid_map = ctx_.atoms_to_grid_idx_map(ia);
 
-            for (auto coord : atom_to_grid_map) {
-                int ir   = coord.first;
-                double r = coord.second;
-                double f = w(Rmt[unit_cell_.atom(ia).type_id()], r);
-                mag(0).rg().value(ir) += v[2] * f;
-                if (ctx_.num_mag_dims() == 3) {
-                    mag(1).rg().value(ir) += v[0] * f;
-                    mag(2).rg().value(ir) += v[1] * f;
+                auto v = unit_cell_.atom(ia).vector_field();
+
+                for (auto coord : atom_to_grid_map) {
+                    int ir   = coord.first;
+                    double r = coord.second;
+                    double f = w(Rmt[unit_cell_.atom(ia).type_id()], r);
+                    mag(0).rg().value(ir) += v[2] * f;
+                    if (ctx_.num_mag_dims() == 3) {
+                        mag(1).rg().value(ir) += v[0] * f;
+                        mag(2).rg().value(ir) += v[1] * f;
+                    }
                 }
             }
         }
@@ -278,8 +299,8 @@ Density::initial_density_full_pot()
     Radial_integrals_rho_free_atom ri(ctx_.unit_cell(), ctx_.pw_cutoff(), 40);
 
     /* compute contribution from free atoms to the interstitial density */
-    auto v = make_periodic_function<index_domain_t::local>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(),
-                                                           [&ri](int iat, double g) { return ri.value(iat, g); });
+    auto v = make_periodic_function<true>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(),
+                                          [&ri](int iat, double g) { return ri.value(iat, g); });
 
     /* initialize density of free atoms (not smoothed) */
     for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
@@ -428,6 +449,10 @@ Density::initial_density_full_pot()
                 }
             }
         }
+        /* synchronize muffin-tin part */
+        for (int iv = 0; iv < ctx_.num_mag_dims(); iv++) {
+            this->component(1 + iv).mt().sync(ctx_.unit_cell().spl_num_atoms());
+        }
     }
 }
 
@@ -1013,7 +1038,6 @@ add_k_point_contribution_dm_pwpp(Simulation_context& ctx__, K_point<T>& kp__, de
     auto bp_coeffs = bp_gen.prepare();
 
     for (int ichunk = 0; ichunk < kp__.beta_projectors().num_chunks(); ichunk++) {
-        // kp__.beta_projectors().generate(ctx__.processing_unit_memory_t(), ichunk);
         bp_gen.generate(bp_coeffs, ichunk);
 
         if (ctx__.num_mag_dims() != 3) {
@@ -1027,7 +1051,7 @@ add_k_point_contribution_dm_pwpp(Simulation_context& ctx__, K_point<T>& kp__, de
 void
 Density::normalize()
 {
-    double nel   = std::get<0>(rho().integrate());
+    double nel   = rho().integrate().total;
     double scale = unit_cell_.num_electrons() / nel;
 
     /* renormalize interstitial part */
@@ -1051,7 +1075,7 @@ Density::check_num_electrons() const
 {
     double nel{0};
     if (ctx_.full_potential()) {
-        nel = std::get<0>(rho().integrate());
+        nel = rho().integrate().total;
     } else {
         nel = rho().rg().f_0().real() * unit_cell_.omega();
     }
@@ -1119,7 +1143,7 @@ Density::generate(K_point_set const& ks__, bool symmetrize__, bool add_core__, b
 
             /* symmetrize density matrix (used in standard uspp case) */
             if (unit_cell_.max_mt_basis_size() != 0) {
-                sirius::symmetrize_density_matrix(unit_cell_, *density_matrix_, ctx_.num_mag_comp());
+                sirius::symmetrize_density_matrix(unit_cell_, ctx_.rotm(), *density_matrix_, ctx_.num_mag_comp());
             }
 
             if (occupation_matrix_) {
@@ -1711,13 +1735,12 @@ Density::generate_valence_mt()
     }
 }
 
-mdarray<double, 2>
-Density::compute_atomic_mag_mom() const
+std::vector<double>
+Density::compute_atomic_mag_mom(int j__) const
 {
     PROFILE("sirius::Density::compute_atomic_mag_mom");
 
-    mdarray<double, 2> mmom({3, unit_cell_.num_atoms()});
-    mmom.zero();
+    std::vector<double> result(unit_cell_.num_atoms(), 0);
 
     #pragma omp parallel for
     for (int ia = 0; ia < unit_cell_.num_atoms(); ia++) {
@@ -1726,51 +1749,35 @@ Density::compute_atomic_mag_mom() const
 
         for (auto coord : atom_to_grid_map) {
             int ir = coord.first;
-            for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-                mmom(j, ia) += mag(j).rg().value(ir);
-            }
+            result[ia] += this->mag(j__).rg().value(ir);
         }
 
-        for (int j : {0, 1, 2}) {
-            mmom(j, ia) *= (unit_cell_.omega() / fft::spfft_grid_size(ctx_.spfft<double>()));
-        }
+        result[ia] *= (unit_cell_.omega() / fft::spfft_grid_size(ctx_.spfft<double>()));
     }
-    mpi::Communicator(ctx_.spfft<double>().communicator()).allreduce(&mmom(0, 0), static_cast<int>(mmom.size()));
-    return mmom;
+    mpi::Communicator(ctx_.spfft<double>().communicator()).allreduce(result.data(), unit_cell_.num_atoms());
+    return result;
 }
 
-std::tuple<std::array<double, 3>, std::array<double, 3>, std::vector<std::array<double, 3>>>
+std::array<periodic_function_integrate_t<double>, 3>
 Density::get_magnetisation() const
 {
     PROFILE("sirius::Density::get_magnetisation");
 
-    std::array<double, 3> total_mag({0, 0, 0});
-    std::vector<std::array<double, 3>> mt_mag(ctx_.unit_cell().num_atoms(), {0, 0, 0});
-    std::array<double, 3> it_mag({0, 0, 0});
+    std::array<periodic_function_integrate_t<double>, 3> result;
+    for (int j = 0; j < 3; j++) {
+        result[j].mt = std::vector<double>(ctx_.unit_cell().num_atoms(), 0);
+    }
 
     std::vector<int> idx = (ctx_.num_mag_dims() == 1) ? std::vector<int>({2}) : std::vector<int>({2, 0, 1});
 
     for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-        auto result       = this->mag(j).integrate();
-        total_mag[idx[j]] = std::get<0>(result);
-        it_mag[idx[j]]    = std::get<1>(result);
-        if (ctx_.full_potential()) {
-            auto v = std::get<2>(result);
-            for (int ia = 0; ia < ctx_.unit_cell().num_atoms(); ia++) {
-                mt_mag[ia][idx[j]] = v[ia];
-            }
+        result[idx[j]] = this->mag(j).integrate();
+        if (!ctx_.full_potential()) {
+            result[idx[j]].mt = this->compute_atomic_mag_mom(j);
         }
     }
 
-    if (!ctx_.full_potential()) {
-        auto mmom = this->compute_atomic_mag_mom();
-        for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-            for (int ia = 0; ia < ctx_.unit_cell().num_atoms(); ia++) {
-                mt_mag[ia][idx[j]] = mmom(j, ia);
-            }
-        }
-    }
-    return std::make_tuple(total_mag, it_mag, mt_mag);
+    return result;
 }
 
 mdarray<double, 2>
@@ -1936,7 +1943,7 @@ Density::mix()
     PROFILE("sirius::Density::mix");
 
     mixer_input();
-    double rms = mixer_->mix(ctx_.cfg().settings().mixer_rms_min());
+    double rms = mixer_->mix(ctx_.cfg().mixer().rms_min());
     mixer_output();
 
     return rms;
@@ -1947,14 +1954,11 @@ Density::print_info(std::ostream& out__) const
 {
     auto result = this->rho().integrate();
 
-    auto total_charge = std::get<0>(result);
-    auto it_charge    = std::get<1>(result);
-    auto mt_charge    = std::get<2>(result);
+    auto total_charge = result.total;
+    auto it_charge    = result.rg;
+    auto mt_charge    = result.mt;
 
     auto result_mag = this->get_magnetisation();
-    auto total_mag  = std::get<0>(result_mag);
-    auto it_mag     = std::get<1>(result_mag);
-    auto mt_mag     = std::get<2>(result_mag);
 
     auto write_vector = [&](r3::vector<double> v__) {
         out__ << "[" << std::setw(9) << std::setprecision(5) << std::fixed << v__[0] << ", " << std::setw(9)
@@ -1977,7 +1981,7 @@ Density::print_info(std::ostream& out__) const
             out__ << std::setw(4) << ia << std::setw(12) << std::setprecision(6) << std::fixed << mt_charge[ia]
                   << std::setw(16) << std::setprecision(6) << std::scientific << core_leakage;
             if (ctx_.num_mag_dims()) {
-                r3::vector<double> v(mt_mag[ia]);
+                r3::vector<double> v({result_mag[0].mt[ia], result_mag[1].mt[ia], result_mag[2].mt[ia]});
                 out__ << "  ";
                 write_vector(v);
                 out__ << std::setw(12) << std::setprecision(6) << std::fixed << v.length();
@@ -1989,7 +1993,7 @@ Density::print_info(std::ostream& out__) const
               << std::endl
               << "interstitial charge   : " << std::setprecision(6) << std::fixed << it_charge << std::endl;
         if (ctx_.num_mag_dims()) {
-            r3::vector<double> v(it_mag);
+            r3::vector<double> v({result_mag[0].rg, result_mag[1].rg, result_mag[2].rg});
             out__ << "interstitial moment   : ";
             write_vector(v);
             out__ << ", magnitude : " << std::setprecision(6) << std::fixed << v.length() << std::endl;
@@ -1999,7 +2003,7 @@ Density::print_info(std::ostream& out__) const
             out__ << "atom                moment                |moment|" << std::endl << hbar(80, '-') << std::endl;
 
             for (int ia = 0; ia < unit_cell_.num_atoms(); ia++) {
-                r3::vector<double> v(mt_mag[ia]);
+                r3::vector<double> v({result_mag[0].mt[ia], result_mag[1].mt[ia], result_mag[2].mt[ia]});
                 out__ << std::setw(4) << ia << " ";
                 write_vector(v);
                 out__ << std::setw(12) << std::setprecision(6) << std::fixed << v.length() << std::endl;
@@ -2010,7 +2014,7 @@ Density::print_info(std::ostream& out__) const
     out__ << "total charge          : " << std::setprecision(6) << std::fixed << total_charge << std::endl;
 
     if (ctx_.num_mag_dims()) {
-        r3::vector<double> v(total_mag);
+        r3::vector<double> v({result_mag[0].total, result_mag[1].total, result_mag[2].total});
         out__ << "total moment          : ";
         write_vector(v);
         out__ << ", magnitude : " << std::setprecision(6) << std::fixed << v.length() << std::endl;
