@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2019 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file density.hpp
  *
@@ -81,7 +70,7 @@ inline auto
 get_rho_up_dn(int num_mag_dims__, double rho__, r3::vector<double> mag__)
 {
     if (rho__ < 0.0) {
-        return std::make_pair<double, double>(0, 0);
+        return std::make_pair<double, double>(0.0, 0.0);
     }
 
     double mag{0};
@@ -345,8 +334,7 @@ class Density : public Field4D
         /* get form-factors for all G shells */
         auto const ff = ctx_.ri().ps_core_->values(q, ctx_.comm());
         /* make rho_core(G) */
-        auto v = make_periodic_function<index_domain_t::local>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(),
-                                                               ff);
+        auto v = make_periodic_function<true>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(), ff);
 
         std::copy(v.begin(), v.end(), &rho_pseudo_core_->f_pw_local(0));
         rho_pseudo_core_->fft_transform(1);
@@ -541,32 +529,32 @@ class Density : public Field4D
     density_matrix_aux(typename atom_index_t::global ia__) const;
 
     /// Calculate approximate atomic magnetic moments in case of PP-PW.
-    mdarray<double, 2>
-    compute_atomic_mag_mom() const;
+    std::vector<double>
+    compute_atomic_mag_mom(int j__) const;
 
     /// Get total magnetization and also contributions from interstitial and muffin-tin parts.
     /** In case of PP-PW there are no real muffin-tins. Instead, a value of magnetization inside atomic
      *  sphere with some chosen radius is returned.
      */
-    std::tuple<std::array<double, 3>, std::array<double, 3>, std::vector<std::array<double, 3>>>
+    std::array<periodic_function_integrate_t<double>, 3>
     get_magnetisation() const;
 
     void
     print_info(std::ostream& out__) const;
 
-    Occupation_matrix const&
+    inline auto const&
     occupation_matrix() const
     {
         return *occupation_matrix_;
     }
 
-    Occupation_matrix&
+    inline auto&
     occupation_matrix()
     {
         return *occupation_matrix_;
     }
 
-    auto const&
+    inline auto const&
     paw_density() const
     {
         return *paw_density_;
@@ -815,7 +803,7 @@ copy(Density const& src__, Density& dest__)
 }
 
 template <bool add_pseudo_core__>
-inline std::array<std::unique_ptr<Smooth_periodic_function<double>>, 2>
+inline auto
 get_rho_up_dn(Density const& density__, double add_delta_rho_xc__ = 0.0, double add_delta_mag_xc__ = 0.0)
 {
     PROFILE("sirius::get_rho_up_dn");
