@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2018 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file dft_ground_state.cpp
  *
@@ -58,7 +47,7 @@ DFT_ground_state::create_H0()
 {
     PROFILE("sirius::DFT_ground_state::create_H0");
 
-    H0_ = std::shared_ptr<Hamiltonian0<double>>(new Hamiltonian0<double>(potential_, true));
+    H0_ = std::make_shared<Hamiltonian0<double>>(potential_, true);
 }
 
 void
@@ -70,10 +59,6 @@ DFT_ground_state::update()
     kset_.update();
     potential_.update();
     density_.update();
-
-    if (!ctx_.full_potential()) {
-        ewald_energy_ = sirius::ewald_energy(ctx_, ctx_.gvec(), ctx_.unit_cell());
-    }
 }
 
 double
@@ -86,13 +71,13 @@ DFT_ground_state::energy_kin_sum_pw() const
 
         #pragma omp parallel for schedule(static) reduction(+:ekin)
         for (int igloc = 0; igloc < kp->num_gkvec_loc(); igloc++) {
-            auto Gk = kp->gkvec().gkvec_cart<index_domain_t::local>(igloc);
+            auto Gk = kp->gkvec().gkvec_cart(gvec_index_t::local(igloc));
 
             double d{0};
             for (int ispin = 0; ispin < ctx_.num_spins(); ispin++) {
                 for (int i = 0; i < kp->num_occupied_bands(ispin); i++) {
-                    double f = kp->band_occupancy(i, ispin);
-                    auto z   = kp->spinor_wave_functions().pw_coeffs(igloc, wf::spin_index(ispin), wf::band_index(i));
+                    auto f = kp->band_occupancy(i, ispin);
+                    auto z = kp->spinor_wave_functions().pw_coeffs(igloc, wf::spin_index(ispin), wf::band_index(i));
                     d += f * (std::pow(z.real(), 2) + std::pow(z.imag(), 2));
                 }
             }
@@ -109,13 +94,13 @@ DFT_ground_state::energy_kin_sum_pw() const
 double
 DFT_ground_state::total_energy() const
 {
-    return sirius::total_energy(ctx_, kset_, density_, potential_, ewald_energy_) + this->scf_correction_energy_;
+    return sirius::total_energy(ctx_, kset_, density_, potential_) + this->scf_correction_energy_;
 }
 
 json
 DFT_ground_state::serialize()
 {
-    return energy_dict(ctx_, kset_, density_, potential_, ewald_energy_, this->scf_correction_energy_);
+    return energy_dict(ctx_, kset_, density_, potential_, this->scf_correction_energy_);
 }
 
 /// A quick check of self-constent density in case of pseudopotential.
@@ -126,7 +111,7 @@ DFT_ground_state::check_scf_density()
         return json();
     }
 
-    auto gs0 = energy_dict(ctx_, kset_, density_, potential_, ewald_energy_);
+    auto gs0 = energy_dict(ctx_, kset_, density_, potential_, this->scf_correction_energy_);
 
     /* create new potential */
     Potential pot(ctx_);
@@ -139,7 +124,8 @@ DFT_ground_state::check_scf_density()
     /* initialize the subspace */
     ::sirius::initialize_subspace(kset_, H0);
     /* find new wave-functions */
-    ::sirius::diagonalize<double, double>(H0, kset_, ctx_.cfg().settings().itsol_tol_min());
+    ::sirius::diagonalize<double, double>(H0, kset_, ctx_.cfg().iterative_solver().min_tolerance(),
+                                          ctx_.cfg().iterative_solver().num_steps());
     /* find band occupancies */
     kset_.find_band_occupancies<double>();
     /* generate new density from the occupied wave-functions */
@@ -148,7 +134,7 @@ DFT_ground_state::check_scf_density()
     Density rho(ctx_);
     rho.generate<double>(kset_, ctx_.use_symmetry(), add_core, transform_to_rg);
 
-    auto gs1 = energy_dict(ctx_, kset_, rho, pot, ewald_energy_);
+    auto gs1 = energy_dict(ctx_, kset_, rho, pot, this->scf_correction_energy_);
 
     auto calc_rms = [&](Field4D& a, Field4D& b) -> double {
         double rms{0};
@@ -211,7 +197,7 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
     s << "density_tol               : " << density_tol__ << std::endl
       << "energy_tol                : " << energy_tol__ << std::endl
       << "iter_solver_tol (initial) : " << iter_solver_tol__ << std::endl
-      << "iter_solver_tol (target)  : " << ctx_.cfg().settings().itsol_tol_min() << std::endl
+      << "iter_solver_tol (target)  : " << ctx_.cfg().iterative_solver().min_tolerance() << std::endl
       << "num_dft_iter              : " << num_dft_iter__;
     ctx_.message(1, __func__, s);
 
@@ -232,9 +218,11 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
             Hamiltonian0<float> H0(potential_, true);
             /* find new wave-functions */
             if (ctx_.cfg().parameters().precision_hs() == "fp32") {
-                result = sirius::diagonalize<float, float>(H0, kset_, iter_solver_tol__);
+                result = sirius::diagonalize<float, float>(H0, kset_, iter_solver_tol__,
+                                                           ctx_.cfg().iterative_solver().num_steps());
             } else {
-                result = sirius::diagonalize<float, double>(H0, kset_, iter_solver_tol__);
+                result = sirius::diagonalize<float, double>(H0, kset_, iter_solver_tol__,
+                                                            ctx_.cfg().iterative_solver().num_steps());
             }
             /* find band occupancies */
             kset_.find_band_occupancies<float>();
@@ -246,7 +234,8 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         } else {
             Hamiltonian0<double> H0(potential_, true);
             /* find new wave-functions */
-            result = sirius::diagonalize<double, double>(H0, kset_, iter_solver_tol__);
+            result = sirius::diagonalize<double, double>(H0, kset_, iter_solver_tol__,
+                                                         ctx_.cfg().iterative_solver().num_steps());
             /* find band occupancies */
             kset_.find_band_occupancies<double>();
             /* generate new density from the occupied wave-functions */
@@ -267,14 +256,14 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
             // tol = rms * rms / std::max(1.0, unit_cell_.num_electrons());
             tol = eha_res / std::max(1.0, unit_cell_.num_electrons());
         }
-        tol = std::min(ctx_.cfg().settings().itsol_tol_scale()[0] * tol,
-                       ctx_.cfg().settings().itsol_tol_scale()[1] * iter_solver_tol__);
+        tol = std::min(ctx_.cfg().iterative_solver().tolerance_scale()[0] * tol,
+                       ctx_.cfg().iterative_solver().tolerance_scale()[1] * iter_solver_tol__);
         /* tolerance can't be too small */
-        iter_solver_tol__ = std::max(ctx_.cfg().settings().itsol_tol_min(), tol);
+        iter_solver_tol__ = std::max(ctx_.cfg().iterative_solver().min_tolerance(), tol);
 
         bool iter_solver_converged{true};
         if (ctx_.cfg().iterative_solver().type() != "exact") {
-            iter_solver_converged = (tol <= ctx_.cfg().settings().itsol_tol_min());
+            iter_solver_converged = (tol <= ctx_.cfg().iterative_solver().min_tolerance());
         }
 
 #if defined(SIRIUS_USE_FP32)
@@ -283,11 +272,11 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
             if (ctx_.cfg().parameters().precision_gs() == "fp64" && ctx_.cfg().parameters().precision_wf() == "fp32") {
                 /* if we reached the mimimum tolerance for fp32 */
                 if ((ctx_.cfg().settings().fp32_to_fp64_rms() == 0 &&
-                     iter_solver_tol__ <= ctx_.cfg().settings().itsol_tol_min()) ||
+                     iter_solver_tol__ <= ctx_.cfg().iterative_solver().min_tolerance()) ||
                     (rms < ctx_.cfg().settings().fp32_to_fp64_rms())) {
                     std::cout << "switching to FP64" << std::endl;
                     ctx_.cfg().unlock();
-                    ctx_.cfg().settings().itsol_tol_min(std::numeric_limits<double>::epsilon() * 10);
+                    ctx_.cfg().iterative_solver().min_tolerance(std::numeric_limits<double>::epsilon() * 10);
                     ctx_.cfg().parameters().precision_wf("fp64");
                     ctx_.cfg().parameters().precision_hs("fp64");
                     ctx_.cfg().lock();
@@ -398,6 +387,16 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
     auto tstop = std::chrono::high_resolution_clock::now();
 
     auto dict = serialize();
+    if (ctx_.num_mag_dims()) {
+        dict["magnetisation"]          = {};
+        auto m                         = density_.get_magnetisation();
+        dict["magnetisation"]["total"] = std::vector<double>({m[0].total, m[1].total, m[2].total});
+        std::vector<std::vector<double>> v;
+        for (int ia = 0; ia < ctx_.unit_cell().num_atoms(); ia++) {
+            v.push_back({m[0].mt[ia], m[1].mt[ia], m[2].mt[ia]});
+        }
+        dict["magnetisation"]["atoms"] = v;
+    }
 
     /* check density */
     if (num_iter >= 0) {
@@ -406,8 +405,8 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         for (int ir = 0; ir < density_.rho().rg().spfft().local_slice_size(); ir++) {
             rho_min = std::min(rho_min, density_.rho().rg().value(ir));
         }
-        dict["rho_min"] = rho_min;
         ctx_.comm().allreduce<double, mpi::op_t::min>(&rho_min, 1);
+        dict["rho_min"] = rho_min;
     }
 
     dict["scf_time"]     = std::chrono::duration_cast<std::chrono::duration<double>>(tstop - tstart).count();
@@ -424,11 +423,6 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         check_scf_density();
     }
 
-    // dict["volume"] = ctx.unit_cell().omega() * std::pow(bohr_radius, 3);
-    // dict["volume_units"] = "angstrom^3";
-    // dict["energy"] = dft.total_energy() * ha2ev;
-    // dict["energy_units"] = "eV";
-
     return dict;
 }
 
@@ -486,7 +480,7 @@ DFT_ground_state::print_info(std::ostream& out__) const
         write_energy2("one-electron contribution", one_elec_en); // eband + deband in QE
         write_energy("hartree contribution", 0.5 * evha);
         write_energy("xc contribution", eexc);
-        write_energy("ewald contribution", ewald_energy_);
+        write_energy("ewald contribution", potential_.ewald_energy());
         write_energy("PAW contribution", potential_.PAW_total_energy(density_));
     }
     write_energy("smearing (-TS)", s_sum);
@@ -497,6 +491,7 @@ DFT_ground_state::print_info(std::ostream& out__) const
         write_energy2("Hubbard one-el contribution", hub_one_elec);
     }
     write_energy2("Total energy", etot);
+    write_energy2("Free energy (E-TS)", etot + s_sum);
     out__ << std::endl;
     write_energy("band gap (eV)", gap);
     write_energy("Efermi", ef);
