@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2019 Anton Kozhevnikov, Ilia Sivkov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file force.cpp
  *
@@ -34,6 +23,12 @@
 #include "hamiltonian/hamiltonian.hpp"
 #include "symmetry/symmetrize_forces.hpp"
 #include "lapw/step_function.hpp"
+#include <string>
+#include <iomanip>
+#include <vector>
+#include <complex>
+#include <algorithm>
+#include <cmath>
 
 namespace sirius {
 
@@ -162,9 +157,14 @@ Force::compute_dmat(K_point<double>* kp__, la::dmatrix<std::complex<double>>& dm
         }
     }
 }
-
 mdarray<double, 2> const&
 Force::calc_forces_total()
+{
+    return calc_forces_total(true /*add scf correction*/);
+}
+
+mdarray<double, 2> const&
+Force::calc_forces_total(bool add_scf_corr)
 {
     forces_total_ = mdarray<double, 2>({3, ctx_.unit_cell().num_atoms()});
     if (ctx_.full_potential()) {
@@ -182,15 +182,22 @@ Force::calc_forces_total()
         calc_forces_nonloc();
         calc_forces_core();
         calc_forces_ewald();
-        calc_forces_scf_corr();
+        if (add_scf_corr) {
+            calc_forces_scf_corr();
+        }
         calc_forces_hubbard();
 
         forces_total_ = mdarray<double, 2>({3, ctx_.unit_cell().num_atoms()});
         for (int ia = 0; ia < ctx_.unit_cell().num_atoms(); ia++) {
             for (int x : {0, 1, 2}) {
-                forces_total_(x, ia) = forces_vloc_(x, ia) + forces_us_(x, ia) + forces_nonloc_(x, ia) +
-                                       forces_core_(x, ia) + forces_ewald_(x, ia) + forces_scf_corr_(x, ia) +
-                                       forces_hubbard_(x, ia);
+                if (add_scf_corr) {
+                    forces_total_(x, ia) = forces_vloc_(x, ia) + forces_us_(x, ia) + forces_nonloc_(x, ia) +
+                                           forces_core_(x, ia) + forces_ewald_(x, ia) + forces_scf_corr_(x, ia) +
+                                           forces_hubbard_(x, ia);
+                } else {
+                    forces_total_(x, ia) = forces_vloc_(x, ia) + forces_us_(x, ia) + forces_nonloc_(x, ia) +
+                                           forces_core_(x, ia) + forces_ewald_(x, ia) + forces_hubbard_(x, ia);
+                }
             }
         }
     }
@@ -274,8 +281,7 @@ Force::calc_forces_hubbard()
 
         for (auto it : kset_.spl_num_kpoints()) {
 
-            auto kp = kset_.get<double>(it.i);
-            // kp->beta_projectors().prepare();
+            auto kp  = kset_.get<double>(it.i);
             auto mg1 = kp->spinor_wave_functions().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
             auto mg2 =
                     kp->hubbard_wave_functions_S().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
@@ -310,34 +316,30 @@ Force::calc_forces_ewald()
 
     double prefac = (ctx_.gvec().reduced() ? 4.0 : 2.0) * (twopi / unit_cell.omega());
 
-    int ig0 = ctx_.gvec().skip_g0();
-
     mdarray<std::complex<double>, 1> rho_tmp({ctx_.gvec().count()});
     rho_tmp.zero();
     #pragma omp parallel for schedule(static)
-    for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
-        int ig = ctx_.gvec().offset() + igloc;
+    for (auto it : skip_g0(ctx_.gvec())) {
 
         std::complex<double> rho(0, 0);
 
         for (int ja = 0; ja < unit_cell.num_atoms(); ja++) {
-            rho += ctx_.gvec_phase_factor(ig, ja) * static_cast<double>(unit_cell.atom(ja).zn());
+            rho += ctx_.gvec_phase_factor(it.ig, ja) * static_cast<double>(unit_cell.atom(ja).zn());
         }
 
-        rho_tmp[igloc] = std::conj(rho);
+        rho_tmp[it.igloc] = std::conj(rho);
     }
 
     #pragma omp parallel for
     for (int ja = 0; ja < unit_cell.num_atoms(); ja++) {
-        for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
-            int ig = ctx_.gvec().offset() + igloc;
+        for (auto it : skip_g0(ctx_.gvec())) {
 
-            double g2 = std::pow(ctx_.gvec().gvec_len<index_domain_t::local>(igloc), 2);
+            double g2 = std::pow(ctx_.gvec().gvec_len(it.igloc), 2);
 
             /* cartesian form for getting cartesian force components */
-            auto gvec_cart = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
+            auto gvec_cart = ctx_.gvec().gvec_cart(it.igloc);
 
-            double scalar_part = prefac * (rho_tmp[igloc] * ctx_.gvec_phase_factor(ig, ja)).imag() *
+            double scalar_part = prefac * (rho_tmp[it.igloc] * ctx_.gvec_phase_factor(it.ig, ja)).imag() *
                                  static_cast<double>(unit_cell.atom(ja).zn()) * std::exp(-g2 / (4 * alpha)) / g2;
 
             for (int x : {0, 1, 2}) {
@@ -431,7 +433,7 @@ Force::calc_forces_us()
                 #pragma omp parallel for schedule(static)
                 for (int igloc = 0; igloc < ctx_.gvec().count(); igloc++) {
                     int ig   = ctx_.gvec().offset() + igloc;
-                    auto gvc = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
+                    auto gvc = ctx_.gvec().gvec_cart(gvec_index_t::local(igloc));
                     for (int ia = 0; ia < atom_type.num_atoms(); ia++) {
                         /* here we write in v_tmp  -i * G * exp[ iGRn] Veff(G)
                          * but in formula we have   i * G * exp[-iGRn] Veff*(G)
@@ -506,7 +508,7 @@ Force::calc_forces_scf_corr()
             int igsh = ctx_.gvec().shell(ig);
 
             /* cartesian form for getting cartesian force components */
-            auto gvec_cart = gvec.gvec_cart<index_domain_t::local>(igloc);
+            auto gvec_cart = gvec.gvec_cart(gvec_index_t::local(igloc));
 
             /* scalar part of a force without multipying by G-vector */
             std::complex<double> z =
@@ -566,7 +568,7 @@ Force::calc_forces_core()
             auto igsh = ctx_.gvec().shell(ig);
 
             /* cartesian form for getting cartesian force components */
-            auto gvec_cart = gvecs.gvec_cart<index_domain_t::local>(igloc);
+            auto gvec_cart = gvecs.gvec_cart(gvec_index_t::local(igloc));
 
             /* scalar part of a force without multipying by G-vector */
             std::complex<double> z = fact * fourpi * ff(igsh, iat) *
@@ -667,9 +669,6 @@ Force::calc_forces_vloc()
 
     auto const& gvecs = ctx_.gvec();
 
-    int gvec_count  = gvecs.gvec_count(ctx_.comm().rank());
-    int gvec_offset = gvecs.gvec_offset(ctx_.comm().rank());
-
     double fact = valence_rho.rg().gvec().reduced() ? 2.0 : 1.0;
 
     /* here the calculations are in lattice vectors space */
@@ -679,16 +678,15 @@ Force::calc_forces_vloc()
 
         int iat = atom.type_id();
 
-        for (int igloc = 0; igloc < gvec_count; igloc++) {
-            int ig   = gvec_offset + igloc;
-            int igsh = ctx_.gvec().shell(ig);
+        for (auto it : gvecs) {
+            int igsh = ctx_.gvec().shell(it.ig);
 
             /* cartesian form for getting cartesian force components */
-            auto gvec_cart = gvecs.gvec_cart<index_domain_t::local>(igloc);
+            auto gvec_cart = gvecs.gvec_cart(it.igloc);
 
             /* scalar part of a force without multiplying by G-vector */
-            std::complex<double> z = fact * fourpi * ff(igsh, iat) * std::conj(valence_rho.rg().f_pw_local(igloc)) *
-                                     std::conj(ctx_.gvec_phase_factor(ig, ia));
+            std::complex<double> z = fact * fourpi * ff(igsh, iat) * std::conj(valence_rho.rg().f_pw_local(it.igloc)) *
+                                     std::conj(ctx_.gvec_phase_factor(it.ig, ia));
 
             /* get force components multiplying by cartesian G-vector  */
             for (int x : {0, 1, 2}) {
@@ -790,11 +788,11 @@ Force::add_ibs_force(K_point<double>* kp__, Hamiltonian_k<double>& Hk__, mdarray
         int iat = type.id();
 
         for (int igk_col = 0; igk_col < kp__->num_gkvec_col(); igk_col++) { // loop over columns
-            auto gvec_col       = kp__->gkvec_col().gvec<index_domain_t::local>(igk_col);
-            auto gkvec_col_cart = kp__->gkvec_col().gkvec_cart<index_domain_t::local>(igk_col);
+            auto gvec_col       = kp__->gkvec_col().gvec(gvec_index_t::local(igk_col));
+            auto gkvec_col_cart = kp__->gkvec_col().gkvec_cart(gvec_index_t::local(igk_col));
             for (int igk_row = 0; igk_row < kp__->num_gkvec_row(); igk_row++) { // for each column loop over rows
-                auto gvec_row       = kp__->gkvec_row().gvec<index_domain_t::local>(igk_row);
-                auto gkvec_row_cart = kp__->gkvec_row().gkvec_cart<index_domain_t::local>(igk_row);
+                auto gvec_row       = kp__->gkvec_row().gvec(gvec_index_t::local(igk_row));
+                auto gkvec_row_cart = kp__->gkvec_row().gkvec_cart(gvec_index_t::local(igk_row));
 
                 int ig12 = ctx_.gvec().index_g12(gvec_row, gvec_col);
 
@@ -811,13 +809,13 @@ Force::add_ibs_force(K_point<double>* kp__, Hamiltonian_k<double>& Hk__, mdarray
 
         for (int x = 0; x < 3; x++) {
             for (int igk_col = 0; igk_col < kp__->num_gkvec_col(); igk_col++) { // loop over columns
-                auto gvec_col = kp__->gkvec_col().gvec<index_domain_t::local>(igk_col);
+                auto gvec_col = kp__->gkvec_col().gvec(gvec_index_t::local(igk_col));
                 for (int igk_row = 0; igk_row < kp__->num_gkvec_row(); igk_row++) { // loop over rows
-                    auto gvec_row = kp__->gkvec_row().gvec<index_domain_t::local>(igk_row);
+                    auto gvec_row = kp__->gkvec_row().gvec(gvec_index_t::local(igk_row));
                     /* compute index of G-G' */
                     int ig12 = ctx_.gvec().index_g12(gvec_row, gvec_col);
                     /* get G-G' */
-                    auto vg = ctx_.gvec().gvec_cart<index_domain_t::global>(ig12);
+                    auto vg = ctx_.gvec().gvec_cart(gvec_index_t::global(ig12));
                     /* multiply by i(G-G') */
                     h1(igk_row, igk_col) = std::complex<double>(0.0, vg[x]) * h(igk_row, igk_col);
                     /* multiply by i(G-G') */
@@ -827,7 +825,7 @@ Force::add_ibs_force(K_point<double>* kp__, Hamiltonian_k<double>& Hk__, mdarray
 
             for (int icol = 0; icol < kp__->num_lo_col(); icol++) {
                 for (int igk_row = 0; igk_row < kp__->num_gkvec_row(); igk_row++) {
-                    auto gkvec_row_cart = kp__->gkvec_row().gkvec_cart<index_domain_t::local>(igk_row);
+                    auto gkvec_row_cart = kp__->gkvec_row().gkvec_cart(gvec_index_t::local(igk_row));
                     /* multiply by i(G+k) */
                     h1(igk_row, icol + kp__->num_gkvec_col()) =
                             std::complex<double>(0.0, gkvec_row_cart[x]) * h(igk_row, icol + kp__->num_gkvec_col());
@@ -839,7 +837,7 @@ Force::add_ibs_force(K_point<double>* kp__, Hamiltonian_k<double>& Hk__, mdarray
 
             for (int irow = 0; irow < kp__->num_lo_row(); irow++) {
                 for (int igk_col = 0; igk_col < kp__->num_gkvec_col(); igk_col++) {
-                    auto gkvec_col_cart = kp__->gkvec_col().gkvec_cart<index_domain_t::local>(igk_col);
+                    auto gkvec_col_cart = kp__->gkvec_col().gkvec_cart(gvec_index_t::local(igk_col));
                     /* multiply by i(G+k) */
                     h1(irow + kp__->num_gkvec_row(), igk_col) =
                             std::complex<double>(0.0, -gkvec_col_cart[x]) * h(irow + kp__->num_gkvec_row(), igk_col);
