@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2021 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file hubbard_matrix.hpp
  *
@@ -89,16 +78,18 @@ class Hubbard_matrix
         return local_;
     }
 
+    /// Return local occupation matrix for a given composite atomic level (atom + local n,l).
     auto&
-    local(int ia__)
+    local(int idx__)
     {
-        return local_[ia__];
+        return local_[idx__];
     }
 
+    /// Return const reference to local occupation matrix block.
     auto const&
-    local(int ia__) const
+    local(int idx__) const
     {
-        return local_[ia__];
+        return local_[idx__];
     }
 
     auto&
@@ -246,25 +237,34 @@ class Hubbard_matrix
         return ctx_;
     }
 
-    int
+    auto
     find_orbital_index(const int ia__, const int n__, const int l__) const
     {
-        int at_lvl = 0;
-        for (at_lvl = 0; at_lvl < static_cast<int>(atomic_orbitals_.size()); at_lvl++) {
-            int lo_ind  = atomic_orbitals_[at_lvl].second;
-            int atom_id = atomic_orbitals_[at_lvl].first;
+        for (int at_lvl = 0; at_lvl < static_cast<int>(atomic_orbitals_.size()); at_lvl++) {
+            int lo_ind = atomic_orbitals_[at_lvl].second;
 
             if ((atomic_orbitals_[at_lvl].first == ia__) &&
-                (ctx_.unit_cell().atom(atom_id).type().lo_descriptor_hub(lo_ind).n() == n__) &&
-                (ctx_.unit_cell().atom(atom_id).type().lo_descriptor_hub(lo_ind).l() == l__))
-                break;
+                (ctx_.unit_cell().atom(ia__).type().lo_descriptor_hub(lo_ind).n() == n__) &&
+                (ctx_.unit_cell().atom(ia__).type().lo_descriptor_hub(lo_ind).l() == l__)) {
+                return at_lvl;
+            }
         }
 
-        if (at_lvl == static_cast<int>(atomic_orbitals_.size())) {
-            std::cout << "atom: " << ia__ << "n: " << n__ << ", l: " << l__ << std::endl;
-            RTE_THROW("Found an arbital that is not listed\n");
+        std::stringstream s;
+        s << "Atomic orbital is not in the list" << std::endl
+          << "  atom: " << ia__ << ", n: " << n__ << ", l: " << l__ << std::endl
+          << "  list of atomic orbitals for a given atom:" << std::endl;
+        for (int at_lvl = 0; at_lvl < static_cast<int>(atomic_orbitals_.size()); at_lvl++) {
+            int lo_ind = atomic_orbitals_[at_lvl].second;
+            if (atomic_orbitals_[at_lvl].first == ia__) {
+                s << "  at_lvl: " << at_lvl
+                  << ", n: " << ctx_.unit_cell().atom(ia__).type().lo_descriptor_hub(lo_ind).n()
+                  << ", l: " << ctx_.unit_cell().atom(ia__).type().lo_descriptor_hub(lo_ind).l() << std::endl;
+            }
         }
-        return at_lvl;
+        RTE_THROW(s);
+
+        return -1;
     }
 };
 
