@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2018 Anton Kozhevnikov, Mathieu Taillefumier, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file k_point.cpp
  *
@@ -101,8 +90,8 @@ K_point<T>::initialize()
 
             fv_eigen_vectors_slab_->zero(memory_t::host, wf::spin_index(0), wf::band_range(0, ctx_.num_fv_states()));
             for (int i = 0; i < ctx_.num_fv_states(); i++) {
-                for (int igloc = 0; igloc < gkvec().gvec_count(comm().rank()); igloc++) {
-                    int ig = igloc + gkvec().gvec_offset(comm().rank());
+                for (int igloc = 0; igloc < gkvec().count(comm().rank()); igloc++) {
+                    int ig = igloc + gkvec().offset(comm().rank());
                     if (ig == i) {
                         fv_eigen_vectors_slab_->pw_coeffs(igloc, wf::spin_index(0), wf::band_index(i)) = 1.0;
                     }
@@ -369,8 +358,8 @@ K_point<T>::generate_gkvec(double gk_cutoff__)
 
     for (int rank = 0; rank < comm().size(); rank++) {
         auto gv = gkvec_->gvec_local(rank);
-        for (int igloc = 0; igloc < gkvec_->gvec_count(rank); igloc++) {
-            int ig       = gkvec_->gvec_offset(rank) + igloc;
+        for (int igloc = 0; igloc < gkvec_->count(rank); igloc++) {
+            int ig       = gkvec_->offset(rank) + igloc;
             auto loc_row = spl_ngk_row.location(ig);
             auto loc_col = spl_ngk_col.location(ig);
             if (loc_row.ib == comm_row().rank()) {
@@ -559,7 +548,7 @@ K_point<T>::save(std::string const& name__, int id__) const
         /* save the order of G-vectors */
         mdarray<int, 2> gv({3, num_gkvec()});
         for (int i = 0; i < num_gkvec(); i++) {
-            auto v = gkvec().template gvec<index_domain_t::global>(i);
+            auto v = gkvec().gvec(gvec_index_t::global(i));
             for (int x : {0, 1, 2}) {
                 gv(x, i) = v[x];
             }
@@ -708,7 +697,7 @@ K_point<T>::generate_atomic_wave_functions(std::vector<int> atoms__,
     #pragma omp parallel for schedule(static)
     for (int igk_loc = 0; igk_loc < this->num_gkvec_loc(); igk_loc++) {
         /* vs = {r, theta, phi} */
-        auto vs = r3::spherical_coordinates(this->gkvec().template gkvec_cart<index_domain_t::local>(igk_loc));
+        auto vs = r3::spherical_coordinates(this->gkvec().gkvec_cart(gvec_index_t::local(igk_loc)));
 
         /* compute real spherical harmonics for G+k vector */
         std::vector<double> rlm(lmmax);
@@ -743,7 +732,7 @@ K_point<T>::generate_atomic_wave_functions(std::vector<int> atoms__,
         std::vector<std::complex<T>> phase_gk(num_gkvec_loc());
         #pragma omp parallel for
         for (int igk_loc = 0; igk_loc < num_gkvec_loc(); igk_loc++) {
-            auto G = gkvec().template gvec<index_domain_t::local>(igk_loc);
+            auto G = gkvec().gvec(gvec_index_t::local(igk_loc));
             /* total phase e^{-i(G+k)r_{\alpha}} */
             phase_gk[igk_loc] = std::conj(static_cast<std::complex<T>>(ctx_.gvec_phase_factor(G, ia)) * phase_k);
         }
