@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2018 Anton Kozhevnikov, Ilia Sivkov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file paw_potential.hpp
  *
@@ -82,20 +71,20 @@ Potential::generate_PAW_effective_potential(Density const& density)
         ae_comp.push_back(&paw_potential_->ae_component(j));
         ps_comp.push_back(&paw_potential_->ps_component(j));
     }
-    sirius::symmetrize_mt_function(unit_cell_.symmetry(), unit_cell_.comm(), ctx_.num_mag_dims(), ae_comp);
-    sirius::symmetrize_mt_function(unit_cell_.symmetry(), unit_cell_.comm(), ctx_.num_mag_dims(), ps_comp);
+    sirius::symmetrize_mt_function(unit_cell_, ctx_.rotm(), ctx_.mpi_grid_mt_sym(), ctx_.num_mag_dims(), ae_comp);
+    sirius::symmetrize_mt_function(unit_cell_, ctx_.rotm(), ctx_.mpi_grid_mt_sym(), ctx_.num_mag_dims(), ps_comp);
 
     /* symmetrize ae- component of Exc */
     paw_ae_exc_->sync(unit_cell_.spl_num_paw_atoms());
     ae_comp.clear();
     ae_comp.push_back(paw_ae_exc_.get());
-    sirius::symmetrize_mt_function(unit_cell_.symmetry(), unit_cell_.comm(), 0, ae_comp);
+    sirius::symmetrize_mt_function(unit_cell_, ctx_.rotm(), ctx_.mpi_grid_mt_sym(), 0, ae_comp);
 
     /* symmetrize ps- component of Exc */
     paw_ps_exc_->sync(unit_cell_.spl_num_paw_atoms());
     ps_comp.clear();
     ps_comp.push_back(paw_ps_exc_.get());
-    sirius::symmetrize_mt_function(unit_cell_.symmetry(), unit_cell_.comm(), 0, ps_comp);
+    sirius::symmetrize_mt_function(unit_cell_, ctx_.rotm(), ctx_.mpi_grid_mt_sym(), 0, ps_comp);
 
     /* calculate PAW Dij matrix */
     #pragma omp parallel for
@@ -127,7 +116,7 @@ Potential::generate_PAW_effective_potential(Density const& density)
 double
 xc_mt_paw(std::vector<XC_functional> const& xc_func__, int lmax__, int num_mag_dims__, SHT const& sht__,
           Radial_grid<double> const& rgrid__, std::vector<Flm const*> rho__, std::vector<double> const& rho_core__,
-          std::vector<Flm>& vxc__, Flm& exclm__)
+          std::vector<Flm>& vxc__, Flm& exclm__, bool use_lapl__)
 {
     int lmmax = sf::lmmax(lmax__);
 
@@ -164,7 +153,7 @@ xc_mt_paw(std::vector<XC_functional> const& xc_func__, int lmax__, int num_mag_d
         vxc.push_back(&vxc__[j]);
     }
 
-    sirius::xc_mt(rgrid__, sht__, xc_func__, num_mag_dims__, rho, vxc, &exclm__);
+    sirius::xc_mt(rgrid__, sht__, xc_func__, num_mag_dims__, rho, vxc, &exclm__, use_lapl__);
     return inner(exclm__, rho0);
 }
 
@@ -206,13 +195,13 @@ Potential::calc_PAW_local_potential(typename atom_index_t::global ia__, std::vec
     }
 
     sirius::xc_mt_paw(xc_func_, l_max, ctx_.num_mag_dims(), *sht_, rgrid, ae_density__, ae_core, vxc,
-                      (*paw_ae_exc_)[ia__]);
+                      (*paw_ae_exc_)[ia__], ctx_.cfg().settings().xc_use_lapl());
     for (int i = 0; i < ctx_.num_mag_dims() + 1; i++) {
         paw_potential_->ae_component(i)[ia__] += vxc[i];
     }
 
     sirius::xc_mt_paw(xc_func_, l_max, ctx_.num_mag_dims(), *sht_, rgrid, ps_density__, ps_core, vxc,
-                      (*paw_ps_exc_)[ia__]);
+                      (*paw_ps_exc_)[ia__], ctx_.cfg().settings().xc_use_lapl());
     for (int i = 0; i < ctx_.num_mag_dims() + 1; i++) {
         paw_potential_->ps_component(i)[ia__] += vxc[i];
     }
