@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2021 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file potential.hpp
  *
@@ -36,7 +25,7 @@ check_xc_potential(Density const& rho__);
 
 double
 xc_mt(Radial_grid<double> const& rgrid__, SHT const& sht__, std::vector<XC_functional> const& xc_func__,
-      int num_mag_dims__, std::vector<Flm const*> rho__, std::vector<Flm*> vxc__, Flm* exc__);
+      int num_mag_dims__, std::vector<Flm const*> rho__, std::vector<Flm*> vxc__, Flm* exc__, bool use_lalp__);
 
 double
 density_residual_hartree_energy(Density const& rho1__, Density const& rho2__);
@@ -145,6 +134,9 @@ class Potential : public Field4D
     /** This is used to verify the variational derivative of Exc w.r.t. magnetisation mag */
     double add_delta_mag_xc_{0};
 
+    /// Ewald energy.
+    double ewald_energy_{0};
+
     void
     init_PAW();
 
@@ -165,8 +157,8 @@ class Potential : public Field4D
                              mdarray<double, 3> const& paw_dij__) const;
 
     /// Compute MT part of the potential and MT multipole moments
-    auto
-    poisson_vmt(Periodic_function<double> const& rho__) const
+    inline auto
+    poisson_vmt(Spheric_function_set<double, atom_index_t> const& rhomt__)
     {
         PROFILE("sirius::Potential::poisson_vmt");
 
@@ -176,9 +168,7 @@ class Potential : public Field4D
         for (auto it : unit_cell_.spl_num_atoms()) {
             auto ia = it.i;
 
-            auto qmt_re = poisson_vmt<false>(
-                    unit_cell_.atom(ia), rho__.mt()[ia],
-                    const_cast<Spheric_function<function_domain_t::spectral, double>&>(hartree_potential_->mt()[ia]));
+            auto qmt_re = poisson_vmt<false>(unit_cell_.atom(ia), rhomt__[ia], hartree_potential_->mt()[ia]);
 
             SHT::convert(ctx_.lmax_rho(), &qmt_re[0], &qmt(0, ia));
         }
@@ -266,8 +256,7 @@ class Potential : public Field4D
         /* get form-factors for all G shells */
         auto const ff = ctx_.ri().vloc_->values(q, ctx_.comm());
         /* make Vloc(G) */
-        auto v = make_periodic_function<index_domain_t::local>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(),
-                                                               ff);
+        auto v = make_periodic_function<true>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(), ff);
 
         std::copy(v.begin(), v.end(), &local_potential_->f_pw_local(0));
 
@@ -283,17 +272,17 @@ class Potential : public Field4D
 
     /// Generate XC potential in the muffin-tins.
     void
-    xc_mt(Density const& density__);
+    xc_mt(Density const& density__, bool use_lapl);
 
     /// Generate non-magnetic XC potential on the regular real-space grid.
     template <bool add_pseudo_core__>
     void
-    xc_rg_nonmagnetic(Density const& density__);
+    xc_rg_nonmagnetic(Density const& density__, bool use_lapl);
 
     /// Generate magnetic XC potential on the regular real-space grid.
     template <bool add_pseudo_core__>
     void
-    xc_rg_magnetic(Density const& density__);
+    xc_rg_magnetic(Density const& density__, bool use_lapl);
 
   public:
     /// Constructor
@@ -305,7 +294,7 @@ class Potential : public Field4D
 
     /// Solve Poisson equation for a single atom.
     template <bool free_atom, typename T>
-    inline std::vector<T>
+    std::vector<T>
     poisson_vmt(Atom const& atom__, Spheric_function<function_domain_t::spectral, T> const& rho_mt__,
                 Spheric_function<function_domain_t::spectral, T>& vha_mt__) const
     {
@@ -917,8 +906,25 @@ class Potential : public Field4D
     {
         return hubbard_potential_;
     }
+
+    inline double
+    ewald_energy() const
+    {
+        return ewald_energy_;
+    }
 };
 
+inline void
+copy(Potential const& src__, Potential& dest__)
+{
+    for (int j = 0; j < src__.ctx().num_mag_dims() + 1; j++) {
+        copy(src__.component(j).rg(), dest__.component(j).rg());
+        if (src__.ctx().full_potential()) {
+            copy(src__.component(j).mt(), dest__.component(j).mt());
+        }
+    }
+}
+
 }; // namespace sirius
 
 #endif // __POTENTIAL_HPP__
