@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2020 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file atom_type.cpp
  *
@@ -24,6 +13,8 @@
 
 #include "atom_type.hpp"
 #include "core/ostream_tools.hpp"
+#include "core/traits.hpp"
+#include <algorithm>
 
 namespace sirius {
 
@@ -312,9 +303,9 @@ Atom_type::print_info(std::ostream& out__) const
 
         out__ << std::endl;
         out__ << "augmented wave basis" << std::endl;
-        for (int j = 0; j < (int)aw_descriptors_.size(); j++) {
+        for (int j = 0; j < static_cast<int>(aw_descriptors_.size()); j++) {
             out__ << "[";
-            for (int order = 0; order < (int)aw_descriptors_[j].size(); order++) {
+            for (int order = 0; order < static_cast<int>(aw_descriptors_[j].size()); order++) {
                 if (order) {
                     out__ << ", ";
                 }
@@ -385,7 +376,7 @@ void
 Atom_type::read_input_core(nlohmann::json const& parser)
 {
     std::string core_str = std::string(parser["core"]);
-    if (int size = (int)core_str.size()) {
+    if (int size = static_cast<int>(core_str.size())) {
         if (size % 2) {
             std::stringstream s;
             s << "wrong core configuration string : " << core_str;
@@ -556,7 +547,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
             // -1/2 or l + 1/2 by changing the sign of l
 
             double j = parser["pseudo_potential"]["beta_projectors"][i]["total_angular_momentum"].get<double>();
-            if (j < (double)l) {
+            if (j < static_cast<double>(l)) {
                 l *= -1;
             }
         }
@@ -576,6 +567,10 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
     d_mtrx.zero();
     auto v = parser["pseudo_potential"]["D_ion"].get<std::vector<double>>();
 
+    if (v.size() != nbf * nbf) {
+        RTE_THROW("wrong size of D_ion");
+    }
+
     for (int i = 0; i < nbf; i++) {
         for (int j = 0; j < nbf; j++) {
             d_mtrx(i, j) = v[j * nbf + i];
@@ -589,7 +584,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
             int j    = parser["pseudo_potential"]["augmentation"][k]["j"].get<int>();
             int l    = parser["pseudo_potential"]["augmentation"][k]["angular_momentum"].get<int>();
             auto qij = parser["pseudo_potential"]["augmentation"][k]["radial_function"].get<std::vector<double>>();
-            if ((int)qij.size() != num_mt_points()) {
+            if (static_cast<int>(qij.size()) != num_mt_points()) {
                 RTE_THROW("wrong size of qij");
             }
             add_q_radial_function(i, j, l, qij);
@@ -605,7 +600,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
         for (size_t k = 0; k < nwf; k++) {
             auto v = dict[k]["radial_function"].get<std::vector<double>>();
 
-            if ((int)v.size() != num_mt_points()) {
+            if (static_cast<int>(v.size()) != num_mt_points()) {
                 std::stringstream s;
                 s << "wrong size of atomic functions for atom type " << symbol_ << " (label: " << label_ << ")"
                   << std::endl
@@ -615,7 +610,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
             }
 
             int l = dict[k]["angular_momentum"].get<int>();
-            int n = -1;
+            int n{-1};
             double occ{0};
             if (dict[k].count("occupation")) {
                 occ = dict[k]["occupation"].get<double>();
@@ -676,7 +671,7 @@ Atom_type::read_pseudo_paw(nlohmann::json const& parser)
         /* read ae wave func */
         auto wfc = parser["pseudo_potential"]["paw_data"]["ae_wfc"][i]["radial_function"].get<std::vector<double>>();
 
-        if ((int)wfc.size() > num_mt_points()) {
+        if (static_cast<int>(wfc.size()) > num_mt_points()) {
             std::stringstream s;
             s << "wrong size of ae_wfc functions for atom type " << symbol_ << " (label: " << label_ << ")" << std::endl
               << "size of ae_wfc radial functions in the file: " << wfc.size() << std::endl
@@ -688,7 +683,7 @@ Atom_type::read_pseudo_paw(nlohmann::json const& parser)
 
         wfc = parser["pseudo_potential"]["paw_data"]["ps_wfc"][i]["radial_function"].get<std::vector<double>>();
 
-        if ((int)wfc.size() > num_mt_points()) {
+        if (static_cast<int>(wfc.size()) > num_mt_points()) {
             std::stringstream s;
             s << "wrong size of ps_wfc functions for atom type " << symbol_ << " (label: " << label_ << ")" << std::endl
               << "size of ps_wfc radial functions in the file: " << wfc.size() << std::endl
@@ -701,13 +696,8 @@ Atom_type::read_pseudo_paw(nlohmann::json const& parser)
 }
 
 void
-Atom_type::read_input(std::string const& str__)
+Atom_type::read_input(nlohmann::json const& parser)
 {
-    auto parser = read_json_from_file_or_string(str__);
-
-    if (parser.empty()) {
-        return;
-    }
 
     if (!parameters_.full_potential()) {
         read_pseudo_uspp(parser);
@@ -748,6 +738,333 @@ Atom_type::read_input(std::string const& str__)
         /* read density */
         free_atom_density_ = parser["free_atom"]["density"].get<std::vector<double>>();
     }
+}
+
+bool
+is_upf_file(std::string const& str__)
+{
+    const std::string ftype = ".upf";
+    auto lcstr              = str__;
+    lcstr                   = trim(lcstr);
+    if (lcstr.size() < ftype.size()) {
+        return false;
+    }
+    return std::equal(ftype.rbegin(), ftype.rend(), lcstr.rbegin(),
+                      [](char a, char b) { return std::tolower(a) == std::tolower(b); });
+}
+
+#ifdef SIRIUS_USE_PUGIXML
+template <typename T>
+std::vector<T>
+vec_from_str(std::string const& str__, identity_t<T> scaling = T{1})
+{
+    std::vector<T> data;
+    std::istringstream iss(str__);
+
+    std::copy(std::istream_iterator<T>(iss), std::istream_iterator<T>(), std::back_inserter(data));
+    std::transform(data.begin(), data.end(), data.begin(), [&scaling](T val) { return val * scaling; });
+
+    return data;
+}
+
+void
+Atom_type::read_pseudo_uspp(pugi::xml_node const& upf)
+{
+    pugi::xml_node header = upf.child("PP_HEADER");
+    symbol_               = header.attribute("element").as_string();
+
+    double zp;
+    zp  = header.attribute("z_valence").as_double();
+    zn_ = int(zp + 1e-10);
+
+    int nmtp = header.attribute("mesh_size").as_int();
+
+    auto rgrid = vec_from_str<double>(upf.child("PP_MESH").child("PP_R").child_value());
+    if (static_cast<int>(rgrid.size()) != nmtp) {
+        RTE_THROW("wrong mesh size");
+    }
+    /* set the radial grid */
+    set_radial_grid(nmtp, rgrid.data());
+
+    local_potential(vec_from_str<double>(upf.child("PP_LOCAL").child_value(), 0.5));
+
+    if (header.attribute("core_correction").as_bool()) {
+        ps_core_charge_density(vec_from_str<double>(upf.child("PP_NLCC").child_value()));
+    } else {
+        ps_core_charge_density(std::vector<double>(rgrid.size(), 0.0));
+    }
+
+    ps_total_charge_density(vec_from_str<double>(upf.child("PP_RHOATOM").child_value()));
+
+    if (local_potential().size() != rgrid.size() || ps_core_charge_density().size() != rgrid.size() ||
+        ps_total_charge_density().size() != rgrid.size()) {
+        std::cout << local_potential().size() << " " << ps_core_charge_density().size() << " "
+                  << ps_total_charge_density().size() << std::endl;
+        RTE_THROW("wrong array size");
+    }
+
+    spin_orbit_coupling_ = header.attribute("has_so").as_bool();
+
+    int nbf = header.attribute("number_of_proj").as_int();
+
+    for (int i = 0; i < nbf; i++) {
+        std::string bstr         = "PP_BETA." + std::to_string(i + 1);
+        pugi::xml_node beta_node = upf.child("PP_NONLOCAL").child(bstr.data());
+        auto beta_raw            = vec_from_str<double>(beta_node.child_value());
+
+        int nr = beta_node.attribute("cutoff_radius_index").as_int();
+        if (nr == 0) {
+            for (int j = beta_raw.size() - 1; j >= 0; j--) {
+                if (abs(beta_raw[j]) > 1.0e-80) {
+                    nr = j + 1;
+                    break;
+                }
+            }
+        }
+        if (nr == 0) {
+            nr = beta_raw.size();
+        }
+
+        std::vector<double> beta(nr);
+        std::copy(beta_raw.begin(), beta_raw.begin() + nr, beta.begin());
+
+        if (static_cast<int>(beta.size()) > num_mt_points()) {
+            std::stringstream s;
+            s << "wrong size of beta functions for atom type " << symbol_ << " (label: " << label_ << ")" << std::endl
+              << "size of beta radial functions in the file: " << beta.size() << std::endl
+              << "radial grid size: " << num_mt_points();
+            RTE_THROW(s);
+        }
+        int l = beta_node.attribute("angular_momentum").as_int();
+        if (spin_orbit_coupling_) {
+            // we encode the fact that the total angular momentum j = l
+            // -1/2 or l + 1/2 by changing the sign of l
+            std::string bstr       = "PP_RELBETA." + std::to_string(i + 1);
+            pugi::xml_node so_node = upf.child("PP_SPIN_ORB").child(bstr.data());
+
+            double j = so_node.attribute("jjj").as_double();
+            if (j < static_cast<double>(l)) {
+                l *= -1;
+            }
+        }
+        // add_beta_radial_function(l, beta);
+        if (spin_orbit_coupling_) {
+            if (l >= 0) {
+                add_beta_radial_function(angular_momentum(l, 1), beta);
+            } else {
+                add_beta_radial_function(angular_momentum(-l, -1), beta);
+            }
+        } else {
+            add_beta_radial_function(angular_momentum(l), beta);
+        }
+    }
+
+    mdarray<double, 2> d_mtrx({nbf, nbf});
+    d_mtrx.zero();
+    auto v = vec_from_str<double>(upf.child("PP_NONLOCAL").child("PP_DIJ").child_value(), 0.5);
+
+    for (int i = 0; i < nbf; i++) {
+        for (int j = 0; j < nbf; j++) {
+            d_mtrx(i, j) = v[j * nbf + i];
+        }
+    }
+    d_mtrx_ion(d_mtrx);
+
+    pugi::xml_node nl_node = upf.child("PP_NONLOCAL");
+    if (!nl_node.child("PP_AUGMENTATION").empty()) {
+        for (int i = 0; i < nbf; i++) {
+            std::string istr     = "PP_BETA." + std::to_string(i + 1);
+            pugi::xml_node inode = nl_node.child(istr.data());
+            int li               = inode.attribute("angular_momentum").as_int();
+
+            for (int j = i; j < nbf; j++) {
+                std::string jstr     = "PP_BETA." + std::to_string(j + 1);
+                pugi::xml_node jnode = nl_node.child(jstr.data());
+                int lj               = jnode.attribute("angular_momentum").as_int();
+
+                for (int l = abs(li - lj); l < li + lj + 1; l++) {
+                    if ((li + lj + l) % 2 != 0) {
+                        continue;
+                    }
+
+                    std::string ijl_str =
+                            "PP_QIJL." + std::to_string(i + 1) + "." + std::to_string(j + 1) + "." + std::to_string(l);
+                    pugi::xml_node ijl_node = nl_node.child("PP_AUGMENTATION").child(ijl_str.data());
+
+                    auto qij = vec_from_str<double>(ijl_node.child_value());
+                    if (static_cast<int>(qij.size()) != num_mt_points()) {
+                        RTE_THROW("wrong size of qij");
+                    }
+                    add_q_radial_function(i, j, l, qij);
+                }
+            }
+        }
+    }
+
+    /* read starting wave functions ( UPF CHI ) */
+    if (!header.attribute("number_of_wfc").empty()) {
+        /* total number of pseudo atomic wave-functions */
+        size_t nwf = header.attribute("number_of_wfc").as_int();
+        /* loop over wave-functions */
+        for (size_t k = 0; k < nwf; k++) {
+            std::string wstr        = "PP_CHI." + std::to_string(k + 1);
+            pugi::xml_node wfc_node = upf.child("PP_PSWFC").child(wstr.data());
+
+            auto v = vec_from_str<double>(wfc_node.child_value());
+
+            if (static_cast<int>(v.size()) != num_mt_points()) {
+                std::stringstream s;
+                s << "wrong size of atomic functions for atom type " << symbol_ << " (label: " << label_ << ")"
+                  << std::endl
+                  << "size of atomic radial functions in the file: " << v.size() << std::endl
+                  << "radial grid size: " << num_mt_points();
+                RTE_THROW(s);
+            }
+
+            int l = wfc_node.attribute("l").as_int();
+            int n = -1;
+            double occ{0};
+            if (!wfc_node.attribute("occupation").empty()) {
+                occ = wfc_node.attribute("occupation").as_double();
+            }
+
+            if (!wfc_node.attribute("label").empty()) {
+                auto c1 = wfc_node.attribute("label").as_string();
+                std::istringstream iss(std::string(1, c1[0]));
+                iss >> n;
+            }
+
+            if (spin_orbit_coupling()) {
+                std::string bstr       = "PP_RELWFC." + std::to_string(k + 1);
+                pugi::xml_node so_node = upf.child("PP_SPIN_ORB").child(bstr.data());
+                if (!so_node.attribute("jchi").empty() && l != 0) {
+
+                    std::string wstr           = "PP_CHI." + std::to_string(k + 2);
+                    pugi::xml_node wfc_node_p1 = upf.child("PP_PSWFC").child(wstr.data());
+
+                    auto v1 = vec_from_str<double>(wfc_node_p1.child_value());
+                    double occ1{0};
+                    if (!wfc_node_p1.attribute("occupation").empty()) {
+                        occ1 = wfc_node_p1.attribute("occupation").as_double();
+                    }
+                    occ += occ1;
+                    for (int ir = 0; ir < num_mt_points(); ir++) {
+                        v[ir] = 0.5 * v[ir] + 0.5 * v1[ir];
+                    }
+                    k += 1;
+                }
+            }
+            add_ps_atomic_wf(n, angular_momentum(l), v, occ);
+        }
+    }
+}
+
+void
+Atom_type::read_pseudo_paw(pugi::xml_node const& upf)
+{
+    is_paw_ = true;
+
+    /* read core energy */
+    if (!upf.child("PP_PAW").attribute("core_energy").empty()) {
+        paw_core_energy(0.5 * upf.child("PP_PAW").attribute("core_energy").as_double());
+    } else {
+        paw_core_energy(0);
+    }
+
+    /* cutoff index */
+    int cutoff_radius_index = upf.child("PP_NONLOCAL").child("PP_AUGMENTATION").attribute("cutoff_r_index").as_int();
+
+    /* read core density and potential */
+    paw_ae_core_charge_density(vec_from_str<double>(upf.child("PP_PAW").child("PP_AE_NLCC").child_value()));
+
+    /* read occupations */
+    paw_wf_occ(vec_from_str<double>(upf.child("PP_PAW").child("PP_OCCUPATIONS").child_value()));
+
+    /* setups for reading AE and PS basis wave functions */
+    int num_wfc = num_beta_radial_functions();
+
+    /* read ae and ps wave functions */
+    for (int i = 0; i < num_wfc; i++) {
+        /* read ae wave func */
+        std::string wstr        = "PP_AEWFC." + std::to_string(i + 1);
+        pugi::xml_node wfc_node = upf.child("PP_FULL_WFC").child(wstr.data());
+        auto wfc                = vec_from_str<double>(wfc_node.child_value());
+
+        if (static_cast<int>(wfc.size()) > num_mt_points()) {
+            std::stringstream s;
+            s << "wrong size of ae_wfc functions for atom type " << symbol_ << " (label: " << label_ << ")" << std::endl
+              << "size of ae_wfc radial functions in the file: " << wfc.size() << std::endl
+              << "radial grid size: " << num_mt_points();
+            RTE_THROW(s);
+        }
+
+        add_ae_paw_wf(std::vector<double>(wfc.begin(), wfc.begin() + cutoff_radius_index));
+
+        wstr     = "PP_PSWFC." + std::to_string(i + 1);
+        wfc_node = upf.child("PP_FULL_WFC").child(wstr.data());
+        wfc      = vec_from_str<double>(wfc_node.child_value());
+
+        if (static_cast<int>(wfc.size()) > num_mt_points()) {
+            std::stringstream s;
+            s << "wrong size of ps_wfc functions for atom type " << symbol_ << " (label: " << label_ << ")" << std::endl
+              << "size of ps_wfc radial functions in the file: " << wfc.size() << std::endl
+              << "radial grid size: " << num_mt_points();
+            RTE_THROW(s);
+        }
+
+        add_ps_paw_wf(std::vector<double>(wfc.begin(), wfc.begin() + cutoff_radius_index));
+    }
+}
+
+void
+Atom_type::read_input(pugi::xml_node const& upf)
+{
+    if (!parameters_.full_potential()) {
+        read_pseudo_uspp(upf);
+
+        std::string pseudo_type = upf.child("PP_HEADER").attribute("pseudo_type").as_string();
+        if (pseudo_type == "PAW") {
+            read_pseudo_paw(upf);
+        }
+    } else {
+        RTE_THROW("Full potential calculations require JSON potential files.");
+    }
+}
+#endif
+
+void
+Atom_type::read_input(std::string const& str__)
+{
+    // Read from SIRIUS json potential file
+    if (!is_upf_file(str__)) {
+        auto parser = read_json_from_file_or_string(str__);
+
+        if (parser.empty()) {
+            return;
+        }
+
+        read_input(parser);
+
+        // Read from standard UPF version 2 xml files
+    } else {
+
+#ifdef SIRIUS_USE_PUGIXML
+        pugi::xml_document doc;
+        pugi::xml_parse_result parser = doc.load_file(str__.data());
+
+        if (!parser) {
+            return;
+        }
+
+        pugi::xml_node upf = doc.child("UPF");
+        if (upf.empty()) {
+            RTE_THROW("SIRIUS can only read UPF files with version >= 2. Use the upf_to_json tool.");
+        }
+        read_input(upf);
+#else
+        RTE_THROW("SIRIUS cannot read UPF files directly without pugixml.")
+#endif
+    }
 
     /* it is already done in input.h; here the different constans are initialized */
     read_hubbard_input();
@@ -838,10 +1155,6 @@ Atom_type::read_hubbard_input()
             if (ho.contains("beta")) {
                 coeff[5] = ho.beta();
             }
-            /* now convert eV in Ha */
-            for (int s = 0; s < 6; s++) {
-                coeff[s] /= ha2ev;
-            }
 
             std::vector<double> initial_occupancy;
 
