@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2020 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file atom_type.hpp
  *
@@ -37,6 +26,10 @@
 #include "core/profiler.hpp"
 #include "core/packed_index.hpp"
 
+#ifdef SIRIUS_USE_PUGIXML
+#include "pugixml.hpp"
+#endif
+
 namespace sirius {
 
 /// Descriptor of a local orbital radial function.
@@ -288,10 +281,25 @@ class Atom_type
     inline void
     read_pseudo_paw(nlohmann::json const& parser);
 
-    /// Read atomic parameters from json file or string.
+    /// Read atomic parameters from json file.
     inline void
     read_input(std::string const& str__);
 
+    inline void
+    read_input(nlohmann::json const& parser);
+
+    /// Read atomic parameters directly from UPF v2 files
+#ifdef SIRIUS_USE_PUGIXML
+    inline void
+    read_pseudo_uspp(pugi::xml_node const& upf);
+
+    inline void
+    read_pseudo_paw(pugi::xml_node const& upf);
+
+    inline void
+    read_input(pugi::xml_node const& upf);
+#endif
+
     /// Initialize descriptors of the augmented-wave radial functions.
     inline void
     init_aw_descriptors()
@@ -428,6 +436,12 @@ class Atom_type
         free_atom_radial_grid_ = std::move(rgrid__);
     }
 
+    inline auto const&
+    atomic_level(int idx) const
+    {
+        return atomic_levels_[idx];
+    }
+
     /// Add augmented-wave descriptor.
     inline void
     add_aw_descriptor(int n, int l, double enu, int dme, int auto_enu)
@@ -802,12 +816,6 @@ class Atom_type
         return static_cast<int>(atomic_levels_.size());
     }
 
-    inline atomic_level_descriptor const&
-    atomic_level(int idx) const
-    {
-        return atomic_levels_[idx];
-    }
-
     inline double
     num_core_electrons() const
     {
@@ -841,6 +849,12 @@ class Atom_type
         free_atom_density_ = rho__;
     }
 
+    inline void
+    aw_default_l(radial_solution_descriptor_set aw_default_l__)
+    {
+        aw_default_l_ = aw_default_l__;
+    }
+
     inline int
     num_aw_descriptors() const
     {
